base_cookie = {
    'user_locale': 'zh',
    'fine_remember_login': '-1',
    'SVRNAME': 'teacher1',
}

header_urlencoded = {
    'origin': 'https://jw.ustc.edu.cn',
    'content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
    'x-requested-with': 'XMLHttpRequest',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.114514.2025 Safari/537.36'
}

header_json = {
    'origin': 'https://jw.ustc.edu.cn',
    'content-type': 'application/json; charset=UTF-8',
    'x-requested-with': 'XMLHttpRequest',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.114514.2025 Safari/537.36'
}

header_uaonly = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.114514.2025 Safari/537.36'
}